#ifndef LIST_H
#define LIST_H

#include "types.h"

#define LIST_OWNS_ITEMS			0x80
#define LIST_ITERATING			0x40
#define LIST_ITERATE_TAIL_TO_HEAD	0x20

typedef enum {
  HEAD_TO_TAIL, TAIL_TO_HEAD
} ListIterationType;

typedef struct ListNode {
  void                    *item;
  struct ListNode         *prev,
                          *next;
} ListNode;

typedef struct List {
  word                     nItems		__attribute__((packed));
  ListNode                *head			__attribute__((packed));
  ListNode                *tail			__attribute__((packed));
  CompFn                  compFn		__attribute__((packed));
  byte                    keyOffset		__attribute__((packed));
  byte                    flags			__attribute__((packed));
  ListNode                *curNode		__attribute__((packed));
} List;

void                     listInit(List * list, CompFn compFn, word keyOffset, Boolean ownsItems);
void                     listDeinit(List * list);
word                     listNitems(List * list);
void                     listIterateInit(List * list, ListIterationType iterationType);
void                    *listIterate(List * list, ListNode ** node);
void                     listIterateDeinit(List * list);
void                    *listPeekHead(List * list, ListNode ** node);
void                    *listPeekTail(List * list, ListNode ** node);
void                     listAddHead(List * list, void *item);
void                     listAddTail(List * list, void *item);
void                     listDel(List * list, ListNode * node);
void                    *listSearch(List * list, ListNode ** node, void *key);
void			*listSearchNext(List * list, ListNode ** node, void *key);

#endif
